from simple_term_menu import TerminalMenu

from .nyaa import Torrent


def ask(l: list[Torrent]):
    items = [
        f'[{get_key(index)}] ' + torrent.name.replace('|', '\\|') + '|' + (
            f'Category: {torrent.category} \\| Date: {torrent.date}'
            f' \\| Size: {torrent.size}'
            '\n'
            f'Seeds: {torrent.seeds} \\| Peers: {torrent.peers}'
            f' \\| Downloads: {torrent.downloads}'
        ) for index, torrent in enumerate(l)
    ]
    menu = TerminalMenu(
            items[:20],
        title='Select a torrent:',
        preview_title='Details',
        preview_size=0.4,
        preview_command=str,
        clear_screen=True,
    )
    index = menu.show()
    if index is None:
        return None
    return l[index]


def get_key(i: int):
    if i < 10:
        return i
    return chr(ord('a') + i - 10)



from datetime import datetime
from dataclasses import dataclass
from bs4 import BeautifulSoup
import requests


@dataclass
class Torrent:
    page_url: str
    torrent_url: str
    magnet_url: str
    date: datetime
    name: str
    category: str
    size: str
    seeds: int
    peers: int
    downloads: int


SUKEBEI_URL = 'https://sukebei.nyaa.si'
NYAA_URL = 'https://nyaa.si'


def lookup(search_words: list[str], sukebei=False) -> list[Torrent]:
    base_url = SUKEBEI_URL if sukebei else NYAA_URL
    response = requests.get(base_url, params={'q': ' '.join(search_words)})
    response.raise_for_status()
    soup = BeautifulSoup(response.content, 'html.parser')
    results = []
    for tr in soup.select('.torrent-list tbody tr'):
        columns = tr.find_all('td')
        link = columns[1].find_all('a')[-1]
        torrent = Torrent(
            category=columns[0].find('a')['title'],
            name=link.string,
            torrent_url=(
                base_url
                + columns[2].select_one('.fa-download').parent['href']
            ),
            magnet_url=columns[2].select_one('.fa-magnet').parent['href'],
            page_url=link['href'],
            size=columns[3].string,
            date=datetime.fromtimestamp(int(columns[4]['data-timestamp'])),
            seeds=int(columns[5].string),
            peers=int(columns[6].string),
            downloads=int(columns[7].string),
        )
        results.append(torrent)
    return results



from argparse import ArgumentParser

from . import cli, executor, nyaa


def main():
    parser = ArgumentParser()
    parser.add_argument('search_words', nargs='+')
    parser.add_argument('-s', '--sukebei', action='store_true')
    parser.add_argument('-c', '--command')
    args = parser.parse_args()
    matches = nyaa.lookup(args.search_words, sukebei=args.sukebei)
    target = cli.ask(matches)
    if not target:
        return
    executor.run(target, command=args.command)


if __name__ == '__main__':
    main()



from pathlib import Path
import os
import appdirs
import yaml

from . import __title__


_default_config = {
    'command': 'webtorrent {}',
}
_config_data = None


def get(key: str) -> str:
    result = get_from_env(key)
    if result is None:
        result = get_from_config(key)
    if result is None:
        raise ValueError(f'Missing config for {key}')
    return result


def get_from_env(key: str):
    env_key = f'{__title__}_{key}'.upper()
    return os.environ.get(env_key)


def get_from_config(key: str):
    global _config_data

    config_key = key.lower()
    path = Path(appdirs.user_config_dir(__title__) + '.yml')

    if not path.exists():
        with path.open('w') as f:
            f.write('\n'.join(
                f'{key}: {value}'
                for key, value in _default_config.items()
            ))
        _config_data = _default_config

    if not _config_data:
        with path.open() as f:
            _config_data = yaml.safe_load(f)

    return _config_data.get(config_key)



from .nyaa import Torrent
from . import config

import subprocess

def run(torrent: Torrent, command=None):
    command = command or config.get('command')
    args = [x.format(torrent.torrent_url) for x in command.split()]
    subprocess.run(args)
